/*
 * Copyright 2002-2005 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.ldaptemplate.support.filter;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.ldaptemplate.support.LdapEncoder;

import org.apache.commons.lang.StringUtils;

/**
 * This filter takes a automatically converts all whitespace to * wildcards.
 * 
 * <pre>
 * WhitespaceWildcardsFilter filter = new WhitespaceWildcardsFilter(&quot;cn&quot;,
 *         &quot;Some CN&quot;);
 * System.out.println(filter.ecode());
 * </pre>
 * 
 * would resut in: <code>(cn=*Some*CN*)</code>
 * 
 * @author Adam Skogman
 * @author Mattias Arthursson
 */
public class WhitespaceWildcardsFilter extends EqualsFilter {
    private static Pattern starReplacePattern = Pattern.compile("\\s+");

    public WhitespaceWildcardsFilter(String attribute, String value) {
        super(attribute, value);
    }

    /**
     * Encodes a value according to the rules for this filter.
     * 
     * @param value
     *            Value to encode.
     * @return Encoded value.
     */
    protected String encodeValue(String value) {

        // blank string means just ONE star
        if (StringUtils.isBlank(value)) {
            return "*";
        }

        // trim value, we will add in stars first and last anywhay
        value = value.trim();

        // filter encode so that any stars etc. are preserved
        String filterEncoded = LdapEncoder.filterEncode(value);

        // Now replace all whitespace with stars
        Matcher m = starReplacePattern.matcher(filterEncoded);

        // possibly 2 longer (stars at ends)
        StringBuffer buff = new StringBuffer(value.length() + 2);

        buff.append('*');

        while (m.find()) {
            m.appendReplacement(buff, "*");
        }
        m.appendTail(buff);

        buff.append('*');

        return buff.toString();
    }
}
